/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __draw_CameraCommon_h__
#define __draw_CameraCommon_h__

namespace fe
{

/**************************************************************************//**
	@brief Adjustable Camera

	@ingroup draw
*//***************************************************************************/
class FE_DL_EXPORT CameraCommon: virtual public CameraI
{
	public:

				CameraCommon(void):
					m_projection(CameraI::e_none),
					m_fov(0.0f,30.0f),
					m_nearplane(0.1f),
					m_farplane(1e4f),
					m_zoom(1.0f),
					m_rasterSpace(FALSE)
				{
					setIdentity(m_cameraMatrix);
					set(m_center);
					setName("CameraCommon");
				}

				CameraCommon(const CameraCommon& rCameraCommon):
					CameraI()
				{	operator=(rCameraCommon);
					setName("CameraCommon"); }

							//* as CameraI
virtual
const	SpatialTransform&	cameraMatrix(void) const
							{	return m_cameraMatrix; }

virtual	Vector2				fov(void) const
							{	return m_fov; }

virtual	CameraI::Projection	projection(void) const
							{	return m_projection; }

		void				setCameraMatrix(const SpatialTransform& matrix)
							{	m_cameraMatrix=matrix; }

		void				setProjection(Projection a_projection)
							{	m_projection=a_projection; }

		void				setPerspective(Real fovy,
									Real nearplane,Real farplane)
							{	set(m_fov,0.0,fovy);
								m_nearplane=nearplane;
								m_farplane=farplane; }
		void				getPerspective(Real& fovy,
									Real& nearplane,Real& farplane)
							{	fovy=m_fov[1];
								nearplane=m_nearplane;
								farplane=m_farplane; }

		void				setOrtho(Real a_zoom, const Vector2 &a_center)
							{	m_zoom = a_zoom;
								m_center = a_center; }
		void				getOrtho(Real &a_zoom, Vector2 &a_center) const
							{	a_zoom = m_zoom;
								a_center = m_center; }

							/** @brief draw to 2D pixel coordinates

								Does not apply the cameraMatrix to the render
								system, but still allows it to be used for
								ViewI::project() and ViewI::unproject().

								*/
		void				setRasterSpace(BWORD a_rasterSpace)
							{	m_rasterSpace = a_rasterSpace; }
		BWORD				rasterSpace(void) const
							{	return m_rasterSpace; }

	protected:

							//* ortho or perspective
		SpatialTransform	m_cameraMatrix;
		CameraI::Projection	m_projection;

							//* perspective
		Vector2				m_fov;
		Real				m_nearplane;
		Real				m_farplane;

							//* ortho
		Vector2				m_center;
		Real				m_zoom;

		BWORD				m_rasterSpace;
};

} // namespace

#endif /* __draw_CameraCommon_h__ */
