/*	Copyright (C) 2003-2015 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __draw_CameraDisplay_h__
#define __draw_CameraDisplay_h__

namespace fe
{

/**************************************************************************//**
	@brief Configuration for a view

	@ingroup draw
*//***************************************************************************/
class FE_DL_EXPORT CameraDisplay: public CameraCommon
{
	public:

						CameraDisplay(void)
						{
							set(m_viewport,0,0,1,1);
							setName("CameraDisplay");
						}

						CameraDisplay(const CameraDisplay& rCameraDisplay):
							CameraCommon()
						{	operator=(rCameraDisplay);
							setName("CameraDisplay"); }

		void			copy(sp<CameraDisplay> spCameraDisplay);

						/// @brief Define region where to draw
		void			setViewport(const Box2i& rect)	{ m_viewport=rect; }
const	Box2i&			viewport(void) const			{ return m_viewport; }

	private:

		Box2i			m_viewport;
};

inline void CameraDisplay::copy(const sp<CameraDisplay> spCameraDisplay)
{
	setCameraMatrix(spCameraDisplay->cameraMatrix());
	setProjection(spCameraDisplay->projection());
	setViewport(spCameraDisplay->viewport());

	Real fovy;
	Real nearplane;
	Real farplane;
	spCameraDisplay->getPerspective(fovy,nearplane,farplane);
	setPerspective(fovy,nearplane,farplane);

	Real zoom;
	Vector2 center;
	spCameraDisplay->getOrtho(zoom,center);
	setOrtho(zoom,center);

	setRasterSpace(spCameraDisplay->rasterSpace());
}

inline String print(const sp<CameraDisplay> spCameraDisplay)
{
	Real fovy;
	Real nearplane;
	Real farplane;
	spCameraDisplay->getPerspective(fovy,nearplane,farplane);

	Real zoom;
	Vector2 center;
	spCameraDisplay->getOrtho(zoom,center);

	const SpatialTransform xform=spCameraDisplay->cameraMatrix();

	String s;
	s.sPrintf("viewport %s fovy %g near %g far %g"
			" zoom %g center %s camera:\n%s",
			c_print(spCameraDisplay->viewport()),
			fovy,nearplane,farplane,
			zoom,c_print(center),c_print(xform));
	return s;
}

} // namespace

#endif /* __draw_CameraDisplay_h__ */
