import sys
import os
import re
import common
forge = sys.modules["forge"]

def setup(module):
	manifest = 'ext/tbb/manifest.cc'
	with open(manifest, 'w') as outfile:
		outfile.write("")

	versions = []

	for tbb_source in reversed(sorted(forge.tbb_sources)):
		if not os.path.exists(tbb_source[2]) or not os.path.exists(tbb_source[3]):
			forge.color_on(0, forge.BLUE)
			sys.stdout.write(" -" + tbb_source[0])
			forge.color_off()
			continue

		tbb_lib = common.find_tbb_lib(tbb_source[3])
		if tbb_lib == '':
			forge.color_on(0, forge.BLUE)
			sys.stdout.write(" -" + tbb_source[0])
			forge.color_off()
			continue

		if tbb_source[0] in versions:
			continue
		versions += [ tbb_source[0] ]

		if tbb_source[0] == "":
			sys.stdout.write(" /")
		else:
			sys.stdout.write(" " + tbb_source[0])

#		sys.stdout.write(" " + tbb_lib)

		srcList = [	"WorkTbb",
					"tbb.pmh",
					"tbbDL" ]

		variant = tbb_source[0]
		if variant != "":
			variantPath = module.modPath + '/' + variant
			if os.path.lexists(variantPath) == 0:
				os.symlink('.', variantPath)

			srcListVariant = []
			for src in srcList:
				srcListVariant += [ variant + '/' + src ]
			srcList = srcListVariant

		dllname = "fexTbb" + variant

		with open(manifest, 'a') as outfile:
			suffix = ""
			if variant != "":
				suffix = "." + variant
			outfile.write('\tspManifest->catalog<String>('+
					'"WorkForceI.WorkTbb.fe'+suffix+'")='+
					'"fexTbb'+variant+'";\n');

		dll = module.DLL( dllname, srcList )

		# icc conflict with Houdini's boost
		dll.cxx = "g++"

		for src in srcList:
			relativePath = forge.RelativePath(forge.rootPath,module.modPath)
			modObjPath = os.path.join(forge.objPath, relativePath)
			objName = modObjPath + '/' + src
			if re.compile(r'(.*)\.pmh$').match(src):
				objName += '.gch'
			else:
				objName += forge.objsuf
			srcTarget = forge.FindTarget(objName)
			srcTarget.includemap = { 'tbb' : tbb_source[2] }
			srcTarget.cxx = dll.cxx

		dll.linkmap = { "tbb" :
				"-L" + tbb_source[3] + ' -l' + tbb_lib }

#		if tbb_source[0] == '':
		dll.linkmap["tbb"] += ' -Xlinker -rpath ' + tbb_source[3]

		deplibs = [ "fePlatformLib" ]

		forge.deps( [ dllname + "Lib" ], deplibs )
