import sys
import os
import re
forge = sys.modules["forge"]

def setup(module):

	version = ""
	for line in open(forge.usd_install + "/include/pxr/imaging/hd/version.h").readlines():
		if re.match("#define HD_API", line):
			version = line.split()[2].rstrip()

	if version != "":
		sys.stdout.write(" " + version)

	deplibs =	forge.corelibs

	module.cppmap = {}
	module.cppmap["std"] = "-std=c++11"
	module.cppmap["usd"] = "-D BUILD_COMPONENT_SRC_PREFIX=\\\"pxr/\\\""
	module.cppmap["usd"] += " -D BUILD_OPTLEVEL_OPT"

	module.includemap = {}

	module.includemap["python"] = "/usr/include/python2.7"

	module.includemap["tbb"] = "/opt/hfs15.0.416/toolkit/include"

	module.includemap["exr"] = module.modPath + "/OpenEXR/include/OpenEXR"
#	module.includemap["exr"] = forge.openexr_source + "/include/OpenEXR"

	module.includemap["usd"] = forge.usd_install + "/include"

	usd_install_lib = forge.usd_install + "/lib"

	module.linkmap = {}
	module.linkmap["usd"] = "-Xlinker -rpath='" + usd_install_lib + "'"
	module.linkmap["usd"] += " -L " + usd_install_lib
	module.linkmap["usd"] += " -lhd -lhdx"

#	module.linkmap["glew"] = "-lGLEW"
#	module.linkmap["python"] = "-lpython2.7"
	module.linkmap["gfxlibs"] = forge.gfxlibs

	srcList = [	"usd.pmh",
				"DrawHydra",
				"HydraDelegate",
				"HydraDriver",
				"TestLightingShader",
				"usdDL" ]

	dll = module.DLL( "fexUsdDL", srcList )

	dll.linkmap = { "gfxlibs": forge.gfxlibs }

	deplibs = forge.corelibs+ [
				"feSignalDLLib",
				"fexDataToolDLLib",
				"fexOpenGLDLLib" ]

	forge.deps( ["fexUsdDLLib"], deplibs )

	forge.tests += [
		("inspect.exe",		"fexUsdDL",				None,		None) ]

	tests = []
#	tests += [	'testHdBasicDrawing' ]

	deplibs = []

	for t in tests:
		exe = module.Exe(t)

		exe.linkmap = {}

		exe.linkmap["usd2"] = ""
		exe.linkmap["usd2"] += " -lar"
		exe.linkmap["usd2"] += " -larch"
		exe.linkmap["usd2"] += " -lgarch"
		exe.linkmap["usd2"] += " -lgf"
		exe.linkmap["usd2"] += " -lglf"
		exe.linkmap["usd2"] += " -ljs"
		exe.linkmap["usd2"] += " -lplug"
		exe.linkmap["usd2"] += " -lpxOsd"
		exe.linkmap["usd2"] += " -lsdf"
		exe.linkmap["usd2"] += " -ltf"
		exe.linkmap["usd2"] += " -ltracelite"
		exe.linkmap["usd2"] += " -lvt"
		exe.linkmap["usd2"] += " -lwork"

		if os.path.exists("ext/usd/USD/install/lib/libglfq.so"):
			exe.linkmap["usd2"] += " -lglfq"

		exe.linkmap["qt"] = "-lQtOpenGL"
		exe.linkmap["qt"] += " -lQtGui"
		exe.linkmap["qt"] += " -lQtCore"

		forge.deps([t + "Exe"], deplibs)

def auto(module):
	test_file = """
#include "pxr/imaging/hd/version.h"

int main(void)
{
	return(0);
}
    \n"""

	includemap = { "usd": forge.usd_install + "/include" }
	linkmap = {}

	return forge.cctest(test_file, linkmap, includemap)
